@@ -3,16 +3,19 @@ from __future__ import unicode_literals
 from typing import Dict, List
 import json
 import uuid
+from pathlib import Path
 from collections import OrderedDict
 from django.contrib.auth import get_user_model
 from django.utils.functional import cached_property
 from django.db import transaction
-from rest_framework import serializers, exceptions, status
+from rest_framework import serializers, exceptions, status, fields
 from vstutils.api import serializers as vst_serializers, fields as vst_fields
+from vstutils.api import auth as vst_auth
 from vstutils.api.serializers import DataSerializer, EmptySerializer
 from vstutils.api.base import Response
-from ...main.utils import AnsibleArgumentsReference, AnsibleInventoryParser
+from ...main.utils import AnsibleArgumentsReference
 from ...main.settings import LANGUAGES
+from ...main.validators import path_validator
 
 from ...main import models
 from ..signals import api_post_save, api_pre_save
@@ -144,20 +147,20 @@ class _SignalSerializer(serializers.ModelSerializer):
     @cached_property
     def _writable_fields(self) -> List:
         writable_fields = super(_SignalSerializer, self)._writable_fields
-        fields = []
+        fields_of_serializer = []
         attrs = [
             'field_name', 'source_attrs', 'source',
             'read_only', 'required', 'write_only', 'default'
         ]
         for field in writable_fields:
             if not isinstance(field, DataSerializer):
-                fields.append(field)
+                fields_of_serializer.append(field)
                 continue
             field_object = serializers.DictField()
             for attr in attrs:
                 setattr(field_object, attr, getattr(field, attr, None))
-            fields.append(field_object)
-        return fields
+            fields_of_serializer.append(field_object)
+        return fields_of_serializer
 
     @with_signals
     def create(self, validated_data):
@@ -183,7 +186,7 @@ class _WithPermissionsSerializer(_SignalSerializer):
         return self.context['request'].user  # noce
 
 
-class UserSerializer(vst_serializers.UserSerializer):
+class UserSerializer(vst_auth.UserSerializer):
     is_staff = serializers.HiddenField(default=True, label='Staff')
 
     @with_signals
@@ -191,14 +194,14 @@ class UserSerializer(vst_serializers.UserSerializer):
         return super(UserSerializer, self).update(instance, validated_data)
 
 
-class CreateUserSerializer(vst_serializers.CreateUserSerializer):
+class CreateUserSerializer(vst_auth.CreateUserSerializer):
 
     @with_signals
     def create(self, validated_data: Dict) -> User:
         return super().create(validated_data)
 
 
-class ChangePasswordSerializer(vst_serializers.ChangePasswordSerializer):
+class ChangePasswordSerializer(vst_auth.ChangePasswordSerializer):
 
     @with_signals
     def update(self, instance: User, validated_data: Dict) -> User:
@@ -208,7 +211,7 @@ class ChangePasswordSerializer(vst_serializers.ChangePasswordSerializer):
 class OneUserSerializer(UserSerializer):
     email = serializers.EmailField(required=False)
 
-    class Meta(vst_serializers.OneUserSerializer.Meta):
+    class Meta(vst_auth.OneUserSerializer.Meta):
         pass
 
 
@@ -471,12 +474,14 @@ class HostSerializer(_WithVariablesSerializer):
         required=False,
         default='HOST'
     )
+    from_project = serializers.BooleanField(read_only=True, label='Project Based')
 
     class Meta:
         model = models.Host
         fields = ('id',
                   'name',
-                  'type',)
+                  'type',
+                  'from_project')
 
 
 class OneHostSerializer(HostSerializer):
@@ -521,7 +526,7 @@ class ModuleSerializer(vst_serializers.VSTSerializer):
 
 
 class OneModuleSerializer(ModuleSerializer):
-    data = DataSerializer()
+    data = fields.JSONField(read_only=True)
 
     class Meta:
         model = models.Module
@@ -725,12 +730,14 @@ class _InventoryOperations(_WithVariablesSerializer):
 
 class GroupSerializer(_WithVariablesSerializer):
     children = serializers.BooleanField(read_only=True)
+    from_project = serializers.BooleanField(read_only=True, label='Project Based')
 
     class Meta:
         model = models.Group
         fields = ('id',
                   'name',
-                  'children',)
+                  'children',
+                  'from_project')
 
 
 class OneGroupSerializer(GroupSerializer, _InventoryOperations):
@@ -756,11 +763,13 @@ class GroupCreateMasterSerializer(OneGroupSerializer):
 
 
 class InventorySerializer(_WithVariablesSerializer):
+    from_project = serializers.BooleanField(read_only=True, label='Project Based')
 
     class Meta:
         model = models.Inventory
         fields = ('id',
-                  'name',)
+                  'name',
+                  'from_project',)
 
 
 class OneInventorySerializer(InventorySerializer, _InventoryOperations):
@@ -808,6 +817,9 @@ class ProjectCreateMasterSerializer(vst_serializers.VSTSerializer, _WithPermissi
                                         label='Branch for GIT(branch/tag/SHA) or TAR(subdir)',
                                         field='type',
                                         types=branch_types)
+    additional_playbook_path = vst_fields.VSTCharField(required=False,
+                                                       allow_null=True,
+                                                       label='Directory with playbooks')
 
     class Meta:
         model = models.Project
@@ -820,16 +832,19 @@ class ProjectCreateMasterSerializer(vst_serializers.VSTSerializer, _WithPermissi
             'repo_auth',
             'auth_data',
             'branch',
+            'additional_playbook_path',
         )
         extra_kwargs = {
             'name': {'required': True}
         }
 
+    @transaction.atomic
     def create(self, validated_data: Dict) -> models.Project:
         repo_type = validated_data.pop('type')
         repo_auth_type = validated_data.pop('repo_auth')
         repo_auth_data = validated_data.pop('auth_data')
         repo_branch = validated_data.pop('branch', None)
+        playbook_path = validated_data.pop('additional_playbook_path', '')
 
         instance = super(ProjectCreateMasterSerializer, self).create(validated_data)
         instance.variables.create(key='repo_type', value=repo_type)
@@ -838,6 +853,8 @@ class ProjectCreateMasterSerializer(vst_serializers.VSTSerializer, _WithPermissi
             instance.variables.create(key=key, value=repo_auth_data)
         if repo_branch:  # nocv
             instance.variables.create(key='repo_branch', value=repo_branch)
+        if playbook_path:
+            instance.variables.create(key='playbook_path', value=playbook_path)
         return instance
 
 
@@ -1002,7 +1019,7 @@ def generate_fileds(ansible_reference: AnsibleArgumentsReference, ansible_type:
     if ansible_type is None:
         return OrderedDict()  # nocv
 
-    fields = OrderedDict()
+    fields_of_serializer = OrderedDict()
 
     for ref, settings in ansible_reference.raw_dict[ansible_type].items():
         if ref in ['help', 'version', ]:
@@ -1036,9 +1053,9 @@ def generate_fileds(ansible_reference: AnsibleArgumentsReference, ansible_type:
                 kwargs['default'] = 'all'
 
         field_name = ref.replace('-', '_')
-        fields[field_name] = field(**kwargs)
+        fields_of_serializer[field_name] = field(**kwargs)
 
-    return fields
+    return fields_of_serializer
 
 
 class AnsibleSerializerMetaclass(serializers.SerializerMetaclass):
@@ -1110,45 +1127,14 @@ class DashboardStatisticSerializer(DataSerializer):
     jobs = DashboardJobsSerializer()
 
 
-class InventoryImportSerializer(DataSerializer):
-    inventory_id = vst_fields.RedirectIntegerField(default=None, allow_null=True)
+class InventoryImportSerializer(serializers.Serializer):
+    # pylint: disable=abstract-method
+    inventory_id = vst_fields.RedirectIntegerField(default=None, allow_null=True, read_only=True)
     name = serializers.CharField(required=True)
     raw_data = vst_fields.VSTCharField()
 
-    @transaction.atomic()
     def create(self, validated_data: Dict) -> Dict:
-        parser = AnsibleInventoryParser()
-        inv_json = parser.get_inventory_data(validated_data['raw_data'])
-
-        inventory = models.Inventory.objects.create(name=validated_data['name'])
-        inventory.vars = inv_json['vars']
-        created_hosts, created_groups = dict(), dict()
-
-        for host in inv_json['hosts']:
-            inv_host = inventory.hosts.create(name=host['name'])
-            inv_host.vars = host['vars']
-            created_hosts[inv_host.name] = inv_host
-
-        for group in inv_json['groups']:
-            children = not len(group['groups']) == 0
-            inv_group = inventory.groups.create(name=group['name'], children=children)
-            inv_group.vars = group['vars']
-            created_groups[inv_group.name] = inv_group
-
-        for group in inv_json['groups']:
-            inv_group = created_groups[group['name']]
-            g_subs = list()
-            if inv_group.children:
-                for name in group['groups']:
-                    g_subs.append(created_groups[name])
-                inv_group.groups.add(*g_subs)
-            else:
-                for name in group['hosts']:
-                    g_subs.append(created_hosts[name])
-                inv_group.hosts.add(*g_subs)
-
-        inventory.raw_data = validated_data['raw_data']
-        return inventory
+        return models.Inventory.import_inventory_from_string(**validated_data)
 
     def to_representation(self, instance):
         return dict(
@@ -1156,3 +1142,20 @@ class InventoryImportSerializer(DataSerializer):
             name=instance.name,
             raw_data=getattr(instance, 'raw_data', '')
         )
+
+
+class InventoryFileImportSerializer(InventoryImportSerializer):
+    name = serializers.CharField(required=True, validators=[path_validator])
+    raw_data = vst_fields.VSTCharField(read_only=True)
+
+    def update(self, instance, validated_data: Dict) -> Dict:
+        inventory_path = Path(instance.path) / Path(validated_data['name'])
+        inventory, _ = instance.slave_inventory.get_or_create(name=inventory_path.stem)
+        inventory.variables.update_or_create(key='inventory_extension', value=inventory_path.suffix, hidden=True)
+        inventory.import_inventory_from_string(
+            raw_data=inventory_path.read_text(),
+            master_project=instance,
+            inventory_instance=inventory,
+            **validated_data
+        )
+        return inventory
